/*
* This file is part of the HdFury Vertex API/DLL control library.

* Vertex and HDFURY are trademark of LegendSky Tech Co Ltd
*
* This program is free to redistribute and/or modify for the purpose of 
* remote controlling HDFURY Vertex device.

* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*
*
* For more information contact:
* LegendSky Tech Co Ltd, Peng Yung Ge Jingli Mansion, 
* No.48 Beili Road South, Shenzhen, PRC
* contact@hdfury.com
* www.hdfury.com
*
*/

#include <stdio.h>
#include "stdafx.h"
#include <wtypes.h>
#include "vertexapi_enum.h"
#include "DefaultEdid.h"
#include "vertexapi.h"  

//#define CEC_TESTS

int main()
{

	unsigned char fwStr[18];
	unsigned char vertexState[64];
	unsigned char pStatus[64];
	unsigned char pAudioStatus[31];
	unsigned char pIf[31], pAviIf[17];
	unsigned char tableEdid[256];

	/* check if Vertex is present */
	if (checkVertexUsb()==true)
		printf("VERTEX found using USB connection\n");
	else {
		printf("Cannot access VERTEX - check USB connection - close other VERTEX software\n");
		return 0; /* exit the program */
	}

	/* Open USB port to Vertex and leave it open*/
	if (usbOpen() == false) {
		printf("Cannot access VERTEX - check USB connection - close other VERTEX software\n");
		return 0;
	}

	if (getVertexVersion(fwStr)) { printf("%s\nStarting tests, press enter to continue\n", fwStr); getchar(); }
	else printf("\nPress ENTER key to Continue\n");

	/* Get the global Vertex setting state array*/
	getVertexStatus(vertexState);

	/* Print some values of the vertexState */
	printf("Current Input port: %s\n",
		(vertexState[eInPort] == INPORT_BOT) ? "BOT" :
		(vertexState[eInPort] == INPORT_TOP) ? "TOP" :
		(vertexState[eInPort] == INPORT_THRU) ? "THRU" : "SWAP");

	/* Check the selected EDID mode. Also check the physical location of the slide switch if requested */
	printf("EDID slide switch is %s position and EDID mode is %s\n",
		(vertexState[eSwState_edid] == SLIDESW_MIDDLE) ? "IN THE MIDDLE" : "NOT IN THE MIDDLE",
		(vertexState[eEdidModeSel] == EDID_AUTOMIX) ? "AUTOMIX" :
		(vertexState[eEdidModeSel] == EDID_FIXED) ? "FIXED" :
		(vertexState[eEdidModeSel] == EDID_CUSTOM) ? "CUSTOM" : "ERROR");
	// If the EDID mode is CUSTOM (slide in the middle), it is possible to modify the value via GUI / RS232. 
	// in this custom mode, the EDIDs can be EDID_AUTOMIX, EDID_FIXED, EDID_CUSTOM, EDID_COPYTOP or EDID_COPYBOT
	// Check the current selected EDID custom mode
	if (vertexState[eEdidModeSel] == EDID_CUSTOM)
		printf("...currently selected CUSTOM EDID mode is %s\n",
			(vertexState[eEdidCustom] == EDID_AUTOMIX) ? "AUTOMIX" :
			(vertexState[eEdidCustom] == EDID_FIXED) ? "FIXED" :
			(vertexState[eEdidCustom] == EDID_CUSTOM) ? "CUSTOM" :
			(vertexState[eEdidCustom] == EDID_COPYTOP) ? "COPYTOP" :
			(vertexState[eEdidCustom] == EDID_COPYBOT) ? "COPYBOT" : "ERROR");


	/* Check the selected EDID mode. Also check the physical location of the slide switch if requested */
	printf("SCALER slide switch is %s position and SCALE mode is %s\n",
		(vertexState[eSwState_scale] == SLIDESW_MIDDLE) ? "IN THE MIDDLE" : "NOT IN THE MIDDLE",
		(vertexState[eScaleModeSel] == SCALE_NONE) ? "NOSCALE" :
		(vertexState[eScaleModeSel] == SCALE_AUTO) ? "AUTO" :
		(vertexState[eScaleModeSel] == SCALE_CUSTOM) ? "CUSTOM" : "ERROR");
	// If the SCALE mode is CUSTOM (slide in the middle), it is possible to modify the value via GUI / RS232. 
	// in this custom mode, the SCALE can be SCALE_NONE, SCALE_AUTO, SCALE_CUSTOM
	// Check the current selected EDID custom mode
	if (vertexState[eScaleModeSel] == SCALE_CUSTOM)
		printf("...currently selected CUSTOM SCALE mode is %s\n",
			(vertexState[eScaleCustom] == SCALE_NONE) ? "NOSCALE" :
			(vertexState[eScaleCustom] == SCALE_AUTO) ? "AUTO" :
			(vertexState[eScaleCustom] == SCALE_CUSTOM) ? "CUSTOM" : "ERROR");

	/* Check the selected HDCP mode. Also check the physical location of the slide switch if requested */
	printf("HDCP slide switch is %s position and HDCP mode is %s\n",
		(vertexState[eSwState_hdcp] == SLIDESW_MIDDLE) ? "IN THE MIDDLE" : "NOT IN THE MIDDLE",
		(vertexState[eEncSel] == HDCP_14) ? "HDCP 1.4" :
		(vertexState[eEncSel] == HDCP_22) ? "HDCP 2.2" : "ERROR");

	/* get status information */
	printf("\nRequest current video/audio information from all connections. Press ENTER key to Continue\n"); getchar();

	if (getStatusRx0(pStatus)) { printf("RX0 [BOT] video info: %s\n", pStatus); }
		else printf("\nCould not execute getStatusRx0()\n");

	if (getStatusRx1(pStatus)) { printf("RX1 [TOP] video info: %s\n", pStatus); }
		else printf("\nCould not execute getStatusRx1()\n");

	if (getStatusTx0(pStatus)) { printf("TX0 [BOT] video info: %s\n", pStatus); }
		else printf("\nCould not execute getStatusTx0()\n");

	if (getStatusTx1(pStatus)) { printf("TX1 [TOP] video info: %s\n", pStatus); }
		else printf("\nCould not execute getStatusTx1()\n");

	if (getStatusTx0Sink(pStatus)) { printf("TX0 [BOT] EDID info: %s\n", pStatus); }
		else printf("\nCould not execute getStatusTx0Sink()\n");

	if (getStatusTx1Sink(pStatus)) { printf("TX1 [TOP] EDID info: %s\n", pStatus); }
		else printf("\nCould not execute getStatusTx1Sink()\n");

	if (getStatusRx0Audio(pAudioStatus)) { printf("RX0 [BOT] AUDIO info: %s\n", pStatus);  }
		else printf("\nCould not execute getStatusRx0Audio()\n");

	if (getStatusRx1Audio(pAudioStatus)) { printf("RX1 [TOP] EDID info: %s\n", pStatus); }
		else printf("\nCould not execute getStatusRx1Audio()\n");


	/* get infoframe information */
	printf("\nRequest current video/audio infoframe information. Press ENTER key to Continue\n"); getchar();

	if (getAviInfoFrame(pAviIf)) {
		char data[64];
		sprintf_s(data, "%02x", pAviIf[0]);
		for (int i = 1; i < 17; i++) {
			sprintf_s(data, "%s:%02x", data, pAviIf[i]);
		}
		printf("AVI: %s\n", data); 
	}
	else printf("\nCould not execute getAviInfoFrame()\n");

	if (getSpdInfoFrame(pIf)) {
		char data[100];
		sprintf_s(data, "%02x", pIf[0]);
		for (int i = 1; i < 31; i++) {
			sprintf_s(data, "%s:%02x", data, pIf[i]);
		}
		printf("SPD: %s\n", data);
	}
	else printf("\nCould not execute getSpdInfoFrame()\n");

	if (getHdrInfoFrame(pIf)) {
		char data[100];
		sprintf_s(data, "%02x", pIf[0]);
		for (int i = 1; i < 31; i++) {
			sprintf_s(data, "%s:%02x", data, pIf[i]);
		}
		printf("HDR: %s\n", data); 
	}
	else printf("\nCould not execute getHdrInfoFrame()\n");

	if (getAudInfoFrame(pIf)) {
		char data[100];
		sprintf_s(data, "%02x", pIf[0]);
		for (int i = 1; i < 31; i++) {
			sprintf_s(data, "%s:%02x", data, pIf[i]);
		}
		printf("AUD: %s\n", data);
	}
	else printf("\nCould not execute getAudInfoFrame()\n");

	if (getHvsInfoFrame(pIf)) {
		char data[100];
		sprintf_s(data, "%02x", pIf[0]);
		for (int i = 1; i < 31; i++) {
			sprintf_s(data, "%s:%02x", data, pIf[i]);
		}
		printf("HVS: %s\n", data);
	}
	else printf("\nCould not execute getHvsInfoFrame()\n");

	if (getVsiInfoFrame(pIf)) {
		char data[100];
		sprintf_s(data, "%02x", pIf[0]);
		for (int i = 1; i < 31; i++) {
			sprintf_s(data, "%s:%02x", data, pIf[i]);
		}
		printf("VSI: %s\n", data);
	}
	else printf("\nCould not execute getVsiInfoFrame()\n");


	/* Execute basic commands */
	printf("Switching to TOP input port. Press Enter to continue\n");  getchar();
	setInputPort(INPORT_TOP);
	printf("Switching to BOT input port. Press Enter to continue\n");  getchar();
	setInputPort(INPORT_BOT);

	printf("Set autoswitch on / priority on and hdcp to 2.2 if slide is in the middle. Press Enter key to Continue\n"); getchar();
	if (vertexState[eSwState_hdcp] == SLIDESW_MIDDLE)
		setHdcpLevel(HDCP_22);  /* can only modify if the slide switch is in the middle */

	setAutoSwitch(STATE_ON);
	setAutoSwitchPrio(STATE_ON);
	setBaudRate(BAUDRATE_9600);

	
	printf("Modify AUTOMIX edid flag values. Make sure EDID slide switch is in the middle. Press Enter key to Continue\n"); getchar();

	if (vertexState[eSwState_edid] == SLIDESW_MIDDLE)
		setEdidMode(EDID_AUTOMIX);  /* can only modify if the slide switch is in the middle */
								
	/* select automix custom EDID */
	setEdidMode(EDID_AUTOMIX);
	/* change EDID algorithm */
	setEdidAlgo(EDIDALGO_MAXALL);
	/* change some EDID flags*/
	setEdidAudioFlags(AUDIO_51);
	setEdidBt2020(STATE_ON);
	setEdidHdr(HDR10_ON_HLG_OFF);
	setEdidDolbyVision(STATE_ON, STATE_OFF, STATE_OFF);

	/* change the custom EDID mode tables - applicable in custom EDID mode only*/
	setEdidCustomTable(2, 100);


	/* Testing the custom EDID writing */
	printf("\nEDID 82 - VERTEX-1080p60-12bit-3D-xvYCC-stereo\n");
	printf("Writing the EDID 82 into the custom slot 5\n");

	if (writeCustomEdid(5, customEdid)) {
		printf("\nWritten the following EDID values...\n");
		for (int i = 0; i<256; i++) {
			if (!(i % 16)) printf("\n");
			printf("%02X ", customEdid[i]);
		}
	}
	else printf("\nError writing EDID values.\n");

	printf("\n\nReading EDID tables from each Vertex input and output. Press Enter key to Continue\n"); getchar();
	printf("Reading the values from Vertex RX0 BOT input. Press Enter key to Continue\n"); 
	if (getEdidRx(0, tableEdid)) { // bottom port
		for (int i = 0; i<256; i++) {
			if (!(i % 16))
				printf("\n"); printf("%02X ", tableEdid[i]);
		}
	}
	else printf("\nError reading edid values.\n");

	printf("\n\nReading the values from Vertex RX1 TOP input\n");
	if (getEdidRx(1, tableEdid)) { // top port
		for (int i = 0; i<256; i++) {
			if (!(i % 16))
				printf("\n"); printf("%02X ", tableEdid[i]);
		}
	}
	else printf("\nError reading edid values.\n");

	printf("\n\nReading the values from Vertex TX0 BOT output\n");
	if (getEdidTx(0, tableEdid)) { // bottom port
		for (int i = 0; i<256; i++) {
			if (!(i % 16))
				printf("\n"); printf("%02X ", tableEdid[i]);
		}
	}
	else printf("\nError reading edid values.\n");

	// read TX1 - top sink EDID
	printf("\n\nReading the EDID from Vertex TX1 TOP output\n");
	if (getEdidTx(1, tableEdid)) { // top port
		for (int i = 0; i<256; i++) {
			if (!(i % 16))
				printf("\n"); printf("%02X ", tableEdid[i]);
		}
	}
	else printf("\nError reading edid values.\n");



	printf("\n\nTest OSD values. Write custom string and modify OSD location. Press Enter key to Continue\n"); getchar();
	setOsdCustomText(400, 40, "HdFury 2018 Test", STATE_ON);

	setOsdParam(STATE_ON,	/* OSD active*/
		10, 800,			/* OSD start coordinates*/
		25, 0, 7, 0x3f,		/* FG color, BG color, BG rate, bitfields*/
		60,					/* Fade time in seconds, 0 = never*/
		STATE_ON,			/* OSD mask active */
		50, 50, 400, 400,	/* OSD mask coordinates */
		5, 10);				/* mask rate, mask grayscale value*/


#ifdef CEC_TESTS

	printf("\nTest CEC: Configure Vertex as AUDIO SYSTEM and issue power off-command. Press Enter to continue\n"); getchar();
	char data[2] = { 0x50, 0x36 }; /* power off command*/

	/* Enable Vertex CEC engine */
	setCecLa(CECLA_VIDEO);	// configure as AUDIO SYSTEM, 
							// must reconfigure CEC if this changes
	setArcHbr(STATE_OFF);

	setCec(STATE_ON);	/* enable CEC*/
	Sleep(2000);  // CEC network propagation takes time - currently this is not optimized
	
	setCecCommand(2, data); 

#endif

	printf("\nTest sending of custom infoframes. Press Enter to continue\n"); getchar();

	printf("\nPopulate new AVI to registers - but do not send to HDMI\n");
	char AVI_Default[17] = { 0x82,0x02,0x0d,0xae,0x40,0xc0,0x60,0x61,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 };
	setAviInfoFrame(AVI_Default);
	setAviCustom(STATE_OFF);
	setAviDisable(STATE_OFF);

	printf("\nPopulate new HDR metadata and start sending it\n");
	char HDRIF_Default[31]; // full size array
	memset(HDRIF_Default, 0, sizeof(HDRIF_Default));
	char HDRIF[30] = { 0x87,0x01,0x1a,0xb0,0x02,0x00,0xc2,0x33,0xc4,0x86,0x4c,0x1d,0xb8,0x0b,0xd0,0x84,0x80,0x3e,0x13,0x3D,0x42,0x40,0xA0,0x0F,0x32,0x00,0xe8,0x03,0x90,0x01 };
	memcpy(HDRIF_Default, HDRIF, sizeof(HDRIF));
	setHdrInfoFrame(HDRIF_Default);
	setHdrCustom(STATE_ON);
	setHdrDisable(STATE_OFF);

	printf("\nPopulate new VSI to registers - but do not send to HDMI\n");
	char VSIIF_Default[31] =
		{ 0x81,0x01,0x1b,0x63,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 };
	setVsiInfoFrame(VSIIF_Default);
	setVsiCustom(STATE_OFF);
	setVsiDisable(STATE_OFF);


	printf("\nTest custom scale parameters. Make sure SCALE slide switch is in the middle for custom modes. Press Enter to continue\n"); getchar();

	getVertexStatus(vertexState); /* refresh the status to see slide switch position */
	if (vertexState[eSwState_scale] == SLIDESW_MIDDLE) {

		setPort0ScalePara(
			SCALE_ON,			
			CS_YUV444_2020,
			DC_12,
			SCALE_ON,
			CS_YUV444_2020,
			DC_12,
			SCALE_ON,
			RESO_4K,
			CS_RGB_FULL,
			DC_12,
			SCALE_ON,
			RESO_4K,
			CS_RGB_FULL,
			DC_12
		);

		setPort1ScalePara(
			SCALE_ON,
			RESO_4K,
			CS_YUV444_2020,
			DC_12,
			SCALE_ON,
			RESO_4K,
			CS_YUV444_2020,
			DC_12,
			SCALE_ON,
			CS_RGB_FULL,
			DC_12,
			SCALE_ON,
			CS_RGB_FULL,
			DC_12
		);


	} else
		printf("\nSCALE slide switch is not in the middle position, cannot continue scaler test\n");


	printf("\n\n\nTests concluded. Press Enter to continue\n"); getchar();

	usbClose();  // close now USB device
	return 0;
}

